import * as t from './types/index';
import { GenerateOptions } from './generator';
export declare type InjectOptions = {
    /**
     * By default all unused props are omitted from the result.
     * Set this to true to include them instead.
     */
    includeUnusedProps?: boolean;
    /**
     * By default existing PropTypes are left alone, set this to true
     * to have them removed before injecting the PropTypes
     */
    removeExistingPropTypes?: boolean;
    /**
     * Used to control which props are includes in the result
     * @return true to include the prop, false to skip it, or undefined to
     * use the default behaviour
     * @default includeUnusedProps ? true : data.usedProps.includes(`${data.prop.name}`)
     */
    shouldInclude?(data: {
        prop: t.PropTypeNode;
        usedProps: string[];
    }): boolean | undefined;
} & Pick<GenerateOptions, 'sortProptypes' | 'includeJSDoc' | 'comment'>;
/**
 * Injects the PropTypes from `parse` into the provided JavaScript code
 * @param propTypes Result from `parse` to inject into the JavaScript code
 * @param target The JavaScript code to add the PropTypes to
 * @param options Options controlling the final result
 */
export declare function inject(propTypes: t.ProgramNode, target: string, options?: InjectOptions): string | null;
