"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var babel = __importStar(require("@babel/core"));
var babelTypes = __importStar(require("@babel/types"));
var generator_1 = require("./generator");
var uuid_1 = require("uuid");
/**
 * Injects the PropTypes from `parse` into the provided JavaScript code
 * @param propTypes Result from `parse` to inject into the JavaScript code
 * @param target The JavaScript code to add the PropTypes to
 * @param options Options controlling the final result
 */
function inject(propTypes, target, options) {
    if (options === void 0) { options = {}; }
    if (propTypes.body.length === 0) {
        return target;
    }
    var propTypesToInject = new Map();
    var result = babel.transformSync(target, {
        plugins: [
            require.resolve('@babel/plugin-syntax-class-properties'),
            require.resolve('@babel/plugin-syntax-jsx'),
            plugin(propTypes, options, propTypesToInject),
        ],
        configFile: false,
        babelrc: false,
        retainLines: true,
    });
    var code = result && result.code;
    if (!code) {
        return null;
    }
    // Replace the placeholders with the generated prop-types
    // Workaround for issues with comments getting removed and malformed
    propTypesToInject.forEach(function (value, key) {
        code = code.replace(key, "\n\n" + value + "\n\n");
    });
    return code;
}
exports.inject = inject;
function plugin(propTypes, options, mapOfPropTypes) {
    if (options === void 0) { options = {}; }
    var _a = options.includeUnusedProps, includeUnusedProps = _a === void 0 ? false : _a, _b = options.removeExistingPropTypes, removeExistingPropTypes = _b === void 0 ? false : _b, otherOptions = __rest(options, ["includeUnusedProps", "removeExistingPropTypes"]);
    var shouldInclude = function (data) {
        if (options.shouldInclude) {
            var result = options.shouldInclude(data);
            if (result !== undefined) {
                return result;
            }
        }
        return includeUnusedProps ? true : data.usedProps.includes("" + data.prop.name);
    };
    var importName = '';
    var needImport = false;
    var alreadyImported = false;
    return {
        visitor: {
            Program: {
                enter: function (path) {
                    if (!path.node.body.some(function (n) {
                        if (babelTypes.isImportDeclaration(n) &&
                            n.source.value === 'prop-types' &&
                            n.specifiers.length) {
                            importName = n.specifiers[0].local.name;
                            alreadyImported = true;
                            return true;
                        }
                    })) {
                        importName = 'PropTypes';
                    }
                    if (removeExistingPropTypes) {
                        path.get('body').forEach(function (nodePath) {
                            var node = nodePath.node;
                            if (babelTypes.isExpressionStatement(node) &&
                                babelTypes.isAssignmentExpression(node.expression, { operator: '=' }) &&
                                babelTypes.isMemberExpression(node.expression.left) &&
                                babelTypes.isIdentifier(node.expression.left.property, { name: 'propTypes' })) {
                                nodePath.remove();
                            }
                        });
                    }
                },
                exit: function (path) {
                    if (alreadyImported || !needImport)
                        return;
                    var propTypesImport = babel.template.ast("import " + importName + " from 'prop-types'");
                    var firstImport = path
                        .get('body')
                        .find(function (nodePath) { return babelTypes.isImportDeclaration(nodePath.node); });
                    // Insert import after the first one to avoid issues with comment flags
                    if (firstImport) {
                        firstImport.insertAfter(propTypesImport);
                    }
                    else {
                        path.node.body = [propTypesImport].concat(path.node.body);
                    }
                },
            },
            FunctionDeclaration: function (path) {
                var node = path.node;
                // Prevent visiting again
                if (node.hasBeenVisited) {
                    path.skip();
                    return;
                }
                if (!node.id)
                    return;
                var props = propTypes.body.find(function (prop) { return prop.name === node.id.name; });
                if (!props)
                    return;
                var usedProps = [];
                if (!includeUnusedProps) {
                    var prop = node.params[0];
                    if (babelTypes.isIdentifier(prop) || babelTypes.isObjectPattern(prop)) {
                        usedProps = getUsedProps(path, prop);
                    }
                }
                if (usedProps.length === 0 && !includeUnusedProps)
                    return;
                needImport = true;
                // Prevent visiting again
                node.hasBeenVisited = true;
                path.skip();
                injectPropTypes({
                    nodeName: node.id.name,
                    usedProps: usedProps,
                    path: path,
                    props: props,
                });
            },
            VariableDeclarator: function (path) {
                var node = path.node;
                // Prevent visiting again
                if (node.hasBeenVisited) {
                    path.skip();
                    return;
                }
                if (!babelTypes.isIdentifier(node.id))
                    return;
                var nodeName = node.id.name;
                var props = propTypes.body.find(function (prop) { return prop.name === nodeName; });
                if (!props)
                    return;
                if (babelTypes.isArrowFunctionExpression(node.init) ||
                    babelTypes.isFunctionExpression(node.init)) {
                    getFromProp(node.init.params[0]);
                }
                // x = react.memo(props => <div/>)
                else if (babelTypes.isCallExpression(node.init)) {
                    var arg = node.init.arguments[0];
                    if (babelTypes.isArrowFunctionExpression(arg) || babelTypes.isFunctionExpression(arg)) {
                        getFromProp(arg.params[0]);
                    }
                }
                function getFromProp(prop) {
                    var usedProps = !includeUnusedProps &&
                        (babelTypes.isIdentifier(prop) || babelTypes.isObjectPattern(prop))
                        ? getUsedProps(path, prop)
                        : [];
                    if (usedProps.length === 0 && !includeUnusedProps)
                        return;
                    needImport = true;
                    // Prevent visiting again
                    node.hasBeenVisited = true;
                    path.skip();
                    injectPropTypes({ path: path.parentPath, usedProps: usedProps, props: props, nodeName: nodeName });
                }
            },
            ClassDeclaration: function (path) {
                var node = path.node;
                // Prevent visiting again
                if (node.hasBeenVisited) {
                    path.skip();
                    return;
                }
                if (!babelTypes.isIdentifier(node.id))
                    return;
                var nodeName = node.id.name;
                var props = propTypes.body.find(function (prop) { return prop.name === nodeName; });
                if (!props)
                    return;
                var usedProps = !includeUnusedProps ? getUsedProps(path, undefined) : [];
                if (usedProps.length === 0 && !includeUnusedProps)
                    return;
                needImport = true;
                // Prevent visiting again
                node.hasBeenVisited = true;
                path.skip();
                injectPropTypes({
                    nodeName: nodeName,
                    usedProps: usedProps,
                    path: path,
                    props: props,
                });
            },
        },
    };
    function injectPropTypes(options) {
        var path = options.path, props = options.props, usedProps = options.usedProps, nodeName = options.nodeName;
        var source = generator_1.generate(props, __assign({}, otherOptions, { importedName: importName, shouldInclude: function (prop) { return shouldInclude({ prop: prop, usedProps: usedProps }); } }));
        var placeholder = "const a" + uuid_1.v4().replace(/\-/g, '_') + " = null;";
        mapOfPropTypes.set(placeholder, source);
        // Insert prop types
        if (babelTypes.isExportNamedDeclaration(path.parent)) {
            path.insertAfter(babel.template.ast("export { " + nodeName + " };"));
            path.insertAfter(babel.template.ast(placeholder));
            path.parentPath.replaceWith(path.node);
        }
        else if (babelTypes.isExportDefaultDeclaration(path.parent)) {
            path.insertAfter(babel.template.ast("export default " + nodeName + ";"));
            path.insertAfter(babel.template.ast(placeholder));
            path.parentPath.replaceWith(path.node);
        }
        else {
            path.insertAfter(babel.template.ast(placeholder));
        }
    }
}
/**
 * Gets used props from path
 * @param rootPath The path to search for uses of rootNode
 * @param rootNode The node to start the search, if undefined searches for `this.props`
 */
function getUsedProps(rootPath, rootNode) {
    var usedProps = [];
    getUsedPropsInternal(rootNode);
    return usedProps;
    function getUsedPropsInternal(node) {
        if (node && babelTypes.isObjectPattern(node)) {
            node.properties.forEach(function (x) {
                if (babelTypes.isObjectProperty(x)) {
                    if (babelTypes.isStringLiteral(x.key)) {
                        usedProps.push(x.key.value);
                    }
                    else {
                        usedProps.push(x.key.name);
                    }
                }
                else if (babelTypes.isIdentifier(x.argument)) {
                    getUsedPropsInternal(x.argument);
                }
            });
        }
        else {
            rootPath.traverse({
                VariableDeclarator: function (path) {
                    var init = path.node.init;
                    if ((node
                        ? babelTypes.isIdentifier(init, { name: node.name })
                        : babelTypes.isMemberExpression(init) &&
                            babelTypes.isThisExpression(init.object) &&
                            babelTypes.isIdentifier(init.property, { name: 'props' })) &&
                        babelTypes.isObjectPattern(path.node.id)) {
                        getUsedPropsInternal(path.node.id);
                    }
                },
                MemberExpression: function (path) {
                    if ((node
                        ? babelTypes.isIdentifier(path.node.object, { name: node.name })
                        : babelTypes.isMemberExpression(path.node.object) &&
                            babelTypes.isMemberExpression(path.node.object.object) &&
                            babelTypes.isThisExpression(path.node.object.object.object) &&
                            babelTypes.isIdentifier(path.node.object.object.property, { name: 'props' })) &&
                        babelTypes.isIdentifier(path.node.property)) {
                        usedProps.push(path.node.property.name);
                    }
                },
            });
        }
    }
}
//# sourceMappingURL=injector.js.map